
import os
import time

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from sface.config import config
from sface.screens.configscreen import ConfigScreen
from sface.screens.mainscreen import MainScreen

class Nav(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        
        self.title = QLabel("", self)
        self.link = QLabel("", self)
        self.link.setAlignment(Qt.AlignRight)
        
        hlayout = QHBoxLayout()
        hlayout.addWidget(self.title)
        hlayout.addStretch()
        hlayout.addWidget(self.link)
        self.setLayout(hlayout)

    def setTitle(self, title):
        self.title.setText(title)

    def setLink(self, link):
        self.link.setText(link)


class Status(QLabel):
    def __init__(self, parent=None):
        QLabel.__init__(self, "", parent)
        self.sliceUpdateDate()

    def set(self, msg, timeout):
        self.setText(msg)
        if timeout:
            QTimer.singleShot(timeout, self.reset)

    def sliceUpdateDate(self):
        rspec_file = config.getSliceRSpecFile()
        if not os.path.exists(rspec_file):
            return

        creation_time = os.stat(rspec_file).st_ctime
        last_update = time.ctime(creation_time)
        self.set("Slice data last updated on %s" % last_update, timeout=None)

    def reset(self):
        self.setText("")
        QTimer.singleShot(1500, self.sliceUpdateDate)

class MainWindow(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

        self.pix = QLabel(self)
        self.config_screen = ConfigScreen(self)
        self.main_screen = MainScreen(self)

        self.screens = QStackedWidget(self)
        self.screens.addWidget(self.main_screen)
        self.screens.addWidget(self.config_screen)
        self.screens.addWidget(self.pix)
        self.next_screen = None

        self.nav = Nav(self)
        self.nav.setTitle(self.main_screen.getTitleText())
        self.nav.setLink(self.config_screen.getLinkText())

        self.status = Status(self)
        self.log = QLabel("<a href='showlog'>Show Log</a>", self)

        hlayout = QHBoxLayout()
        hlayout.addWidget(self.status)
        hlayout.addStretch()
        hlayout.addWidget(self.log)

        layout = QVBoxLayout()
        layout.addWidget(self.nav)
        layout.addWidget(self.screens)
        layout.addLayout(hlayout)
        self.setLayout(layout)
        self.resize(800, 600)

        self.connect(self.nav.link, SIGNAL('linkActivated(QString)'),
                     self.animateToScreen)


    def animatePixmap(self, y):
        self.pix.move(0, y)

    def animateToScreen(self, link):
        if link == self.config_screen.name:
            self.next_screen = self.config_screen
        elif link == self.main_screen.name:
            self.next_screen = self.main_screen
        
        pixmap = QPixmap(self.screens.size())
        self.screens.currentWidget().render(pixmap)
        self.screens.setCurrentWidget(self.pix)
        self.pix.setPixmap(pixmap)

        timeLine = QTimeLine(500, self)
        timeLine.setFrameRange(0, self.screens.height());
        self.connect(timeLine, SIGNAL('frameChanged(int)'), self.animatePixmap)
        self.connect(timeLine, SIGNAL('finished()'), self.toNextScreen)
        timeLine.start()


    def toNextScreen(self):
        if self.next_screen == self.config_screen:
            self.toConfigScreen()
        elif self.next_screen == self.main_screen:
            self.toMainScreen()

    def toConfigScreen(self):
        self.screens.setCurrentWidget(self.config_screen)
        self.nav.setLink(self.main_screen.getLinkText())
        self.nav.setTitle(self.config_screen.getTitleText())

    def toMainScreen(self):
        self.screens.setCurrentWidget(self.main_screen)
        self.nav.setLink(self.config_screen.getLinkText())
        self.nav.setTitle(self.main_screen.getTitleText())

    def setStatus(self, msg, timeout):
        self.status.set(msg, timeout)
